/**
* @license Apache-2.0
*
* Copyright (c) 2018 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

'use strict';

/**
* Determine the quarter of the year.
*
* @module @stdlib/time/quarter-of-year
*
* @example
* var quarterOfYear = require( '@stdlib/time/quarter-of-year' );
*
* var q = quarterOfYear( new Date() );
* // returns <number>
*
* q = quarterOfYear( 4 );
* // returns 2
*
* q = quarterOfYear( 'June' );
* // returns 2
*/

// MODULES //

var quarterOfYear = require( './quarter_of_year.js' );


// EXPORTS //

module.exports = quarterOfYear;
