/*
* @license Apache-2.0
*
* Copyright (c) 2019 The Stdlib Authors.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*    http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

// TypeScript Version: 2.0

/**
* Returns a high-resolution time difference, where `time` is a two-element array with format `[seconds, nanoseconds]`.
*
* ## Notes
*
* -   Similar to `time`, the returned array has format `[seconds, nanoseconds]`.
*
*
* @param time - high-resolution time
* @throws must provide a nonnegative integer array
* @throws input array must have length `2`
* @returns high resolution time difference
*
* @example
* var tic = require( `@stdlib/time/tic` );
*
* var start = tic();
* var delta = toc( start );
* // returns [<number>,<number>]
*/
declare function toc( time: Array<number> ): Array<number>;


// EXPORTS //

export = toc;
